﻿(* Raima.Rdm is the system module that contains the ADO.NET data provider
   for RDM *)
open Raima.Rdm

(* EXAMPLE - SQL03 for F# using ADO.NET *)
(* When you run this example, a new database is created with three tables
 * in a heirarchy where one references another, which refernces the third
 * one.  Then a single row is created in the top table and three rows are
 * created in the middle table and nine rows in the bottom table.
 * Finally all the data is read from the database using a join.
 * 
 * The following methods are illustrated:
 *     Raima.Rdm.RdmConnection:
 *         Close() Method
 *         CreateCommand() Method
 *         Open() Method
 *         RdmConnection(String) Constructor
 *     Raima.Rdm.RdmCommand:
 *         CommandText Property
 *         ExecuteNonQuery() Method
 *         ExecuteReader() Method
 *     Raima.Rdm.RdmDataReader:
 *         Item[int] Property
 *         Read() Method
 * 
 * They are fully document in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown exceptions
 * but good programming practices would dictate that this occur in
 * a full application.
 *)

let CreateDatabase =
    let conn = new RdmConnection("host=local")
    conn.Open()

    let cmd = conn.CreateCommand()

    (* Attempt to drop the database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. *)
    try
        [|
            "DROP DATABASE SQL03";
            "COMMIT"
        |] |>
            Array.iter(fun c ->
                cmd.CommandText <- c
                cmd.ExecuteNonQuery() |> ignore
            )
    with
        | exn -> ()

    (* Attempt to create the database *)
    [|
        "CREATE DATABASE SQL03";
        "CREATE TABLE TopTab (tid INTEGER PRIMARY KEY, tstr CHAR(30))";
        "CREATE TABLE MidTab (mid INTEGER PRIMARY KEY, mstr CHAR(30), tid INTEGER REFERENCES TopTab(tid))";
        "CREATE TABLE BotTab (bstr CHAR(30), mid INTEGER REFERENCES MidTab(mid))";
        "COMMIT"
    |] |>
        Array.iter(fun c ->
            cmd.CommandText <- c
            cmd.ExecuteNonQuery() |> ignore
        )

    conn.Close()

CreateDatabase

(* Create the connection with a simple connection string *)
let conn = new RdmConnection("host=local;database=SQL03;openmode=x")

(* Open the connection to the database *)
conn.Open()

(* Create a new command *)
let cmd = conn.CreateCommand()

(* Insert a row into the top table *)
cmd.CommandText <- "INSERT INTO TopTab VALUES 1, 'top 1'"
cmd.ExecuteNonQuery() |> ignore

(* Insert three rows into the middle table *)
[1..3] |>
    List.iter(fun ii ->
        cmd.CommandText <- "INSERT INTO MidTab VALUES " + (string ii) + ", 'mid " + (string ii) + "', 1"
        cmd.ExecuteNonQuery() |> ignore
        [1..3] |>
            List.iter(fun jj ->
                cmd.CommandText <- "INSERT INTO BotTab VALUES 'bot " + (string ((ii - 1) * 3 + jj)) + "', " + (string ii)
                cmd.ExecuteNonQuery() |> ignore
        )
    )

(* Scan through all the existing rows using a join and print out the contents *)
cmd.CommandText <- "SELECT tstr, mstr, bstr FROM TopTab, MidTab, BotTab WHERE TopTab.tid = MidTab.tid AND MidTab.mid = BotTab.mid"
let reader = cmd.ExecuteReader()

while (reader.Read()) do
    printfn "%s -- %s -- %s" (string (reader.Item(0))) (string (reader.Item(1))) (string (reader.Item(2)))

conn.Close()

printfn "Hit <Enter> to continue"
System.Console.ReadLine() |> ignore

